// { dg-options "-std=gnu++11" }
// { dg-do compile }

// Copyright (C) 2011-2016 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or (at your option)
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License along
// with this library; see the file COPYING3.  If not see
// <http://www.gnu.org/licenses/>.

// NB: This file is for testing memory with NO OTHER INCLUDES.

#include <memory>

template<typename T>
struct P1
{
  using element_type = T;
  using difference_type = long;
  template<typename U> using rebind = P1<U>;
  static P1 pointer_to(T&) { return {}; }
};

template<typename T>
struct P2
{
  static P2 pointer_to(T&) { return {}; }
};

namespace std
{
  typedef short test_type;
  template struct pointer_traits<test_type*>;
  template struct pointer_traits<P1<test_type>>;
  template struct pointer_traits<P2<test_type>>;
}
